package resource

// Request represents generic named resource (bucket, container etc.).
// Name is resource depenent but should be globally unique for any given
// type of resource.
type Request interface {
	// Name is the operation name, such as Object.Put. Must not include wildcards.
	Operation() string
	// Property returns request properties, such as IP address of the origin.
	Property(string) string
	// Resource returns resource the operation is applied to.
	Resource() Resource
}

// Resource represents the resource operation is applied to.
type Resource interface {
	Name() string
	Property(string) string
}
